from django.contrib.admin import AdminSite as BaseAdminSite
from django.contrib.auth import authenticate
from django.forms import forms

__all__ = ("AdminSite",)


class LoginForm(forms.Form):
    def __init__(self, request=None, *args, **kwargs):
        self.request = request
        self.user_cache = None
        super().__init__(*args, **kwargs)

    def clean(self):
        self.user_cache = authenticate(self.request)
        return self.cleaned_data

    def get_user(self):
        return self.user_cache


class AdminSite(BaseAdminSite):
    login_form = LoginForm
    login_template = "admin/login_without_credentials.html"



from django.urls import path

from . import views

app_name = "posts"
urlpatterns = [
    path("", views.QuillPostListView.as_view(), name="quill-post-list"),
    path("create/", views.QuillPostCreateView.as_view(), name="quill-post-create"),
    path(
        "create/normal/",
        views.QuillFieldFormView.as_view(),
        name="quill-post-create-normal",
    ),
    path(
        "create/normal/html/",
        views.QuillFieldFormHtmlInitialView.as_view(),
        name="quill-post-create-normal-html",
    ),
    path(
        "create/normal/text/",
        views.QuillFieldFormTextInitialView.as_view(),
        name="quill-post-create-normal-text",
    ),
    path("<int:pk>/", views.QuillPostDetailView.as_view(), name="quill-post-detail"),
    path(
        "<int:pk>/update/",
        views.QuillPostUpdateView.as_view(),
        name="quill-post-update",
    ),
]



from django.conf import settings
from django.shortcuts import redirect
from django.urls import reverse_lazy, reverse
from django.views import View
from django.views.generic import CreateView

from posts.forms import QuillPostForm, QuillFieldForm
from posts.models import QuillPost

__all__ = (
    "IndexView",
    "ResetView",
)


class IndexView(CreateView):
    template_name = "index.html"
    form_class = QuillPostForm
    success_url = reverse_lazy("posts:quill-post-list")

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["normal_form"] = QuillFieldForm()
        context["title_img"] = settings.TITLE_IMG
        return context


class ResetView(View):
    def post(self, request):
        QuillPost.objects.all().delete()
        return redirect(reverse("posts:quill-post-list"))



from django.db import models
from django.urls import reverse

from django_quill.fields import QuillField

__all__ = ("QuillPost",)


class QuillPost(models.Model):
    content = QuillField()

    class Meta:
        ordering = ["-pk"]

    def get_absolute_url(self):
        return reverse("posts:quill-post-detail", args=[self.pk])



from django import forms
from .widgets import QuillWidget

__all__ = ("QuillFormField",)


class QuillFormField(forms.fields.CharField):
    def __init__(self, *args, **kwargs):
        kwargs.update(
            {
                "widget": QuillWidget(),
            }
        )
        super().__init__(*args, **kwargs)



import json

from django.utils.functional import LazyObject
from model_bakery import baker as default_baker

__all__ = ("baker",)


class LazyBaker(LazyObject):
    @staticmethod
    def _gen_quill_field():
        return json.dumps(
            {
                "delta": {
                    "ops": [
                        {"insert": "Django", "attributes": {"bold": True}},
                        {"insert": " Quill "},
                        {
                            "insert": "Editor",
                            "attributes": {"bold": True, "color": "#ff0000"},
                        },
                    ]
                },
                "html": '<p><strong>Django</strong> Quill <strong style="color: #ff0000;">Editor</strong></p>',
            }
        )

    def _setup(self):
        default_baker.generators.add(
            "django_quill.fields.QuillField", self._gen_quill_field
        )
        self._wrapped = default_baker


baker = LazyBaker()



# Generated by Django 3.1.7 on 2021-03-22 03:15

from django.db import migrations, models
import django_quill.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="QuillPost",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("content", django_quill.fields.QuillField()),
            ],
            options={
                "ordering": ["-pk"],
            },
        ),
    ]



from django.contrib.auth import get_user_model

User = get_user_model()

__all__ = ("StaticBackend",)


class StaticBackend:
    def authenticate(self, request):
        user = User.objects.get_or_create(
            username="django-quill-editor",
            is_staff=True,
            is_superuser=True,
        )[0]
        return user

    def get_user(self, user_id):
        try:
            user = User.objects.get(id=user_id)
            return user
        except User.DoesNotExist:
            return None



DEFAULT_CONFIG = {
    "theme": "snow",
    "modules": {
        "syntax": True,
        "toolbar": [
            [
                {"font": []},
                {"header": []},
                {"align": []},
                "bold",
                "italic",
                "underline",
                "strike",
                "blockquote",
                {"color": []},
                {"background": []},
            ],
            ["code-block", "link", "image"],
            ["clean"],
        ],
        # quill-image-compress
        "imageCompressor": {
            "quality": 0.7,
            "maxWidth": 2000,
            "maxHeight": 2000,
            "imageType": "image/webp",
            "debug": False,
            "suppressErrorLogging": True,
        },
    },
}



from django import forms

from django_quill.forms import QuillFormField
from .models import QuillPost

__all__ = (
    "QuillFieldForm",
    "QuillPostForm",
)


class QuillFieldForm(forms.Form):
    content = QuillFormField()

    def save(self):
        return QuillPost.objects.create(content=self.cleaned_data["content"])


class QuillPostForm(forms.ModelForm):
    class Meta:
        model = QuillPost
        fields = ("content",)



from django.conf import settings
from django.urls import reverse
from django.views.generic import ListView, UpdateView, DetailView, FormView, CreateView

from .forms import QuillFieldForm, QuillPostForm
from .models import QuillPost

__all__ = (
    "QuillPostListView",
    "QuillPostCreateView",
    "QuillPostDetailView",
    "QuillPostUpdateView",
    "QuillFieldFormView",
    "QuillFieldFormHtmlInitialView",
    "QuillFieldFormTextInitialView",
)

HTML_INITIAL = f"""
<h1><strong>django-quill-editor</strong></h1>
<br>
<p>django-quill-editor makes&nbsp;Quill.js&nbsp;easy to use on Django Forms and admin sites</p>
<ul>
    <li><strong>No configuration required for static files!</strong></li>
    <li>The entire code for inserting WYSIWYG editor is less than 30 lines</li>
    <li>It can be used in both admin and Django views</li>
</ul>
<p>
<img src="{settings.TITLE_IMG}"></p>
<ul>
    <li><a href="https://django-quill-editor.readthedocs.io/en/latest/pages/using-in-admin.html" rel="noopener noreferrer" target="_blank" style="color: rgb(41, 128, 185);">Using in Django admin</a></li>
    <li><a href="https://django-quill-editor.readthedocs.io/en/latest/pages/using-as-form.html" rel="noopener noreferrer" target="_blank" style="color: rgb(41, 128, 185);">Using as Form</a></li>
    <li><a href="https://django-quill-editor.readthedocs.io/en/latest/pages/using-as-modelform.html" rel="noopener noreferrer" target="_blank" style="color: rgb(41, 128, 185);">Using as ModelForm</a></li>
    <li><a href="https://django-quill-editor.readthedocs.io/en/latest/pages/change-toolbar-configs.html" rel="noopener noreferrer" target="_blank" style="color: rgb(41, 128, 185);">Change toolbar config</a></li>
</ul>
<br>
<h2><strong>Installation</strong></h2>
<p>Use pip to install from PyPI:</p>
<pre class="ql-syntax" spellcheck="false">
    <span class="hljs-attribute">pip</span> install django-quill-editor
</pre>
<br>
<p>Add&nbsp;<strong>django_quill</strong>&nbsp;to&nbsp;<strong>INSTALLED_APPS</strong>&nbsp;in&nbsp;<strong>settings.py</strong>:</p>
<pre class="ql-syntax" spellcheck="false">
<span class="hljs-attr">INSTALLED_APPS</span> = [
    <span class="hljs-string">'django.contrib.admin'</span>,
    ...
    <span class="hljs-string">'django_quill'</span>,
]
</pre>
<br>
<h2><strong>Contributing</strong></h2>
<p>To contribute to&nbsp;<strong>django-quill-editor</strong>&nbsp;<a href="https://github.com/leehanyeong/django-quill-editor" rel="noopener noreferrer" target="_blank" style="color: rgb(41, 128, 185);">create a fork</a>&nbsp;on GitHub. Clone your fork, make some changes, and submit a pull request.</p>
<br>
<h2><strong>Issues</strong></h2>
<p>Use the GitHub&nbsp;<a href="https://github.com/leehanyeong/django-quill-editor/issues" rel="noopener noreferrer" target="_blank" style="color: rgb(41, 128, 185);">issue tracker</a>&nbsp;for&nbsp;<strong>django-quill-editor</strong>&nbsp;to submit bugs, issues, and feature requests.</p>"""


class QuillPostListView(ListView):
    model = QuillPost


class QuillPostCreateView(CreateView):
    form_class = QuillPostForm
    template_name = "posts/quillpost_create.html"


class QuillPostDetailView(DetailView):
    model = QuillPost


class QuillPostUpdateView(UpdateView):
    model = QuillPost
    fields = ["content"]


# NormalForm
class QuillFieldFormView(FormView):
    form_class = QuillFieldForm
    template_name = "posts/quillpost_normal_form.html"

    def form_valid(self, form):
        self.object = form.save()
        return super().form_valid(form)

    def get_success_url(self):
        return reverse("posts:quill-post-detail", args=(self.object.pk,))


class QuillFieldFormHtmlInitialView(QuillFieldFormView):
    initial = {
        "content": HTML_INITIAL,
    }


class QuillFieldFormTextInitialView(QuillFieldFormView):
    initial = {
        "content": "django-quill-editor makes Quill.js easy to use on Django Forms and admin sites",
    }



"""
Django settings for django-quill-editor-playground project.

Generated by 'django-admin startproject' using Django 3.1.7.
"""

from pathlib import Path

BASE_DIR = Path(__file__).resolve().parent.parent

SECRET_KEY = "er8-=^6ym+!t&r#4-=3cq3!25%3hw=9n+5bp1i2549ltqfg=xp"
DEBUG = True
ALLOWED_HOSTS = []

# Static
STATIC_DIR = BASE_DIR / "static"
STATIC_ROOT = BASE_DIR / ".static"
STATIC_URL = "/static/"
STATICFILES_DIRS = [STATIC_DIR]
MEDIA_URL = "/media/"
MEDIA_ROOT = BASE_DIR / ".media"

# Custom
DATA_UPLOAD_MAX_MEMORY_SIZE = 10 * 1024 * 1024  # 10MB
DEFAULT_AUTO_FIELD = "django.db.models.AutoField"
AUTHENTICATION_BACKENDS = [
    "config.backends.StaticBackend",
]
TITLE_IMG = (
    "https://raw.githubusercontent.com/LeeHanYeong/django-quill-editor/"
    "master/_assets/django-quill-editor-sample.png"
)

INSTALLED_APPS = [
    "posts",
    "config.apps.AdminConfig",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django_quill",
]

ROOT_URLCONF = "config.urls"
WSGI_APPLICATION = "config.wsgi.application"
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": BASE_DIR / "db.sqlite3",
    }
}

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [BASE_DIR / "templates"],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]

LANGUAGE_CODE = "en-us"
TIME_ZONE = "UTC"
USE_I18N = True
USE_L10N = True
USE_TZ = True



import json

__all__ = (
    "QuillParseError",
    "Quill",
)


class QuillParseError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return "Failed to parse value(%s)" % self.value


class Quill:
    def __init__(self, json_string):
        try:
            self.json_string = json_string
            json_data = json.loads(json_string)
            self.delta = json_data["delta"]
            self.html = json_data.get("html", "")
        except (json.JSONDecodeError, KeyError, TypeError):
            raise QuillParseError(json_string)



#!/usr/bin/env python
import os

from setuptools import find_packages, setup

ROOT_DIR = os.path.dirname(os.path.abspath(__file__))
README = open(os.path.join(ROOT_DIR, "README.md")).read()
VERSION = open(os.path.join(ROOT_DIR, "version.txt")).read()

setup(
    name="django-quill-editor",
    version=VERSION,
    url="https://github.com/LeeHanYeong/django-quill-editor",
    author="lhy",
    author_email="dev@lhy.kr",
    license="MIT",
    description="Integrate Quill editor with Django project.",
    long_description=README,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["test*", "sample"]),
    include_package_data=True,
    install_requires=[
        "django>=3.0",
    ],
    python_requires=">3.6",
    zip_safe=True,
    classifiers=[
        "Framework :: Django",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Development Status :: 4 - Beta",
    ],
)



from django.db import models

from .forms import QuillFormField
from .quill import Quill

__all__ = (
    "FieldQuill",
    "QuillDescriptor",
    "QuillField",
)


class FieldQuill:
    def __init__(self, instance, field, json_string):
        self.instance = instance
        self.field = field
        self.json_string = json_string or '{"delta":"","html":""}'
        self._committed = True

    def __eq__(self, other):
        if hasattr(other, "json_string"):
            return self.json_string == other.json_string
        return self.json_string == other

    def __hash__(self):
        return hash(self.json_string)

    def _require_quill(self):
        if not self:
            raise ValueError(
                "The '%s' attribute has no Quill JSON String associated with it."
                % self.field.name
            )

    def _get_quill(self):
        self._require_quill()
        self._quill = Quill(self.json_string)
        return self._quill

    def _set_quill(self, quill):
        self._quill = quill

    def _del_quill(self):
        del self._quill

    quill = property(_get_quill, _set_quill, _del_quill)

    @property
    def html(self):
        self._require_quill()
        return self.quill.html

    @property
    def delta(self):
        self._require_quill()
        return self.quill.delta

    def save(self, json_string, save=True):
        setattr(self.instance, self.field.name, json_string)
        self._committed = True
        if save:
            self.instance.save()


class QuillDescriptor:
    def __init__(self, field):
        self.field = field

    def __get__(self, instance, cls=None):
        if instance is None:
            return self

        if self.field.name in instance.__dict__:
            quill = instance.__dict__[self.field.name]
        else:
            instance.refresh_from_db(fields=[self.field.name])
            quill = getattr(instance, self.field.name)

        if isinstance(quill, str) or quill is None:
            attr = self.field.attr_class(instance, self.field, quill)
            instance.__dict__[self.field.name] = attr

        elif isinstance(quill, Quill) and not isinstance(quill, FieldQuill):
            quill_copy = self.field.attr_class(instance, self.field, quill.json_string)
            quill_copy.quill = quill
            quill_copy._committed = False
            instance.__dict__[self.field.name] = quill_copy

        elif isinstance(quill, FieldQuill) and not hasattr(quill, "field"):
            quill.instance = instance
            quill.field = self.field

        elif isinstance(quill, FieldQuill) and instance is not quill.instance:
            quill.instance = instance

        return instance.__dict__[self.field.name]

    def __set__(self, instance, value):
        instance.__dict__[self.field.name] = value


class QuillField(models.TextField):
    attr_class = FieldQuill
    descriptor_class = QuillDescriptor

    def formfield(self, **kwargs):
        kwargs.update({"form_class": QuillFormField})
        return super().formfield(**kwargs)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @staticmethod
    def _get_form_class():
        return QuillFormField

    def from_db_value(self, value, expression, connection):
        return self.to_python(value)

    def to_python(self, value):
        """
        Expect a JSON string with 'delta' and 'html' keys
        ex) b'{"delta": "...", "html": "..."}'
        :param value: JSON string with 'delta' and 'html' keys
        :return: Quill's 'Delta' JSON String
        """
        if isinstance(value, Quill):
            return value
        if isinstance(value, FieldQuill):
            return value.quill
        if value is None or isinstance(value, str):
            return value
        return Quill(value)

    def get_prep_value(self, value):
        value = super().get_prep_value(value)
        if value is None:
            return value
        if isinstance(value, Quill):
            return value.json_string
        return value

    def value_to_string(self, obj):
        value = self.value_from_object(obj)
        return self.get_prep_value(value)



from django.test import TestCase
from django.urls import reverse

from config import baker
from posts.models import QuillPost


class QuillViewTest(TestCase):
    def test_generic_list_view(self):
        baker.make(QuillPost, _quantity=30)
        response = self.client.get(reverse("posts:quill-post-list"))
        self.assertEqual(response.status_code, 200)

    def test_generic_create_view(self):
        response = self.client.get(reverse("posts:quill-post-create"))
        self.assertEqual(response.status_code, 200)

    def test_generic_detail_view(self):
        quill_post = baker.make(QuillPost)
        response = self.client.get(
            reverse("posts:quill-post-detail", args=[quill_post.id])
        )
        self.assertEqual(response.status_code, 200)


class QuillAdminTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.posts = baker.make(QuillPost, _quantity=30)

    def setUp(self) -> None:
        self.client.login()

    def test_list(self):
        response = self.client.get(reverse("admin:posts_quillpost_changelist"))
        self.assertEqual(response.status_code, 200)

    def test_detail(self):
        response = self.client.get(
            reverse("admin:posts_quillpost_change", args=[self.posts[0].id])
        )
        self.assertEqual(response.status_code, 200)



from collections.abc import Mapping

from django import forms
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.serializers.json import DjangoJSONEncoder
from django.forms.renderers import get_default_renderer
from django.forms.utils import flatatt
from django.utils.encoding import force_text
from django.utils.functional import Promise
from django.utils.safestring import mark_safe

from .config import DEFAULT_CONFIG

__all__ = (
    "LazyEncoder",
    "QuillWidget",
)


class LazyEncoder(DjangoJSONEncoder):
    def default(self, obj):
        if isinstance(obj, Promise):
            return force_text(obj)
        return super(LazyEncoder, self).default(obj)


json_encode = LazyEncoder().encode


class QuillWidget(forms.Textarea):
    class Media:
        js = (
            "https://cdnjs.cloudflare.com/ajax/libs/highlight.js/10.1.1/highlight.min.js",
            "django_quill/django_quill.js",
            "https://cdn.quilljs.com/1.3.7/quill.min.js",
        )
        css = {
            "all": (
                "https://cdnjs.cloudflare.com/ajax/libs/highlight.js/10.1.1/styles/darcula.min.css",
                "django_quill/django_quill.css",
                "https://cdn.quilljs.com/1.3.7/quill.snow.css",
            )
        }

    def __init__(self, config_name="default", *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.config = DEFAULT_CONFIG.copy()
        configs = getattr(settings, "QUILL_CONFIGS", None)
        if configs:
            if isinstance(configs, Mapping):
                if config_name in configs:
                    config = configs[config_name]
                    if not isinstance(config, Mapping):
                        raise ImproperlyConfigured(
                            'QUILL_CONFIGS["%s"] setting must be a Mapping object'
                            % config_name
                        )
                    self.config.update(config)
                else:
                    raise ImproperlyConfigured(
                        'No configuration named "%s" found in your QUILL_CONFIGS'
                        % config_name
                    )
            else:
                raise ImproperlyConfigured(
                    "QUILL_CONFIGS settings must be a Mapping object"
                )

    def render(self, name, value, attrs=None, renderer=None):
        if renderer is None:
            renderer = get_default_renderer()
        if value is None:
            value = ""

        attrs = attrs or {}
        attrs["name"] = name
        if hasattr(value, "quill"):
            attrs["quill"] = value.quill
        else:
            attrs["value"] = value
        final_attrs = self.build_attrs(self.attrs, attrs)
        return mark_safe(
            renderer.render(
                "django_quill/widget.html",
                {
                    "final_attrs": flatatt(final_attrs),
                    "id": final_attrs["id"],
                    "name": final_attrs["name"],
                    "config": json_encode(self.config),
                    "quill": final_attrs.get("quill", None),
                    "value": final_attrs.get("value", None),
                },
            )
        )
